# -*- coding: utf-8 -*-

'''some function
   @author Dr. Wang Liang
'''

import math
import numpy as np
import numpy.polynomial as poly


def bv_to_teff(bv):
    ''' B-V to teff
    '''
    coeff = [3.979145106714099,
             -0.654992268598245,
             1.740690042385095,
             -4.608815154057166,
             6.792599779944473,
             -5.396909891322525,
             2.192970376522490,
             -0.359495739295671,
             ]
    f = poly.Polynomial(coeff)
    return math.pow(10, f(bv))


def teff_to_rgb(temp):
    '''
    convert black-body temperature to RGB color
    '''
    t = temp / 1000.

    # calculate red
    if t < 6.527:
        red = 1.0
    else:
        coeff = [4.93596077e+00, -1.29917429e+00,
                 1.64810386e-01, -1.16449912e-02,
                 4.86540872e-04, -1.19453511e-05,
                 1.59255189e-07, -8.89357601e-10]
        tt = min(t, 40)
        f = poly.Polynomial(coeff)
        red = f(tt)
    red = max(red, 0)
    red = min(red, 1)

    # calcuate green
    if t < 0.85:
        green = 0.0
    elif t < 6.6:
        coeff = [-4.95931720e-01, 1.08442658e+00,
                 -9.17444217e-01, 4.94501179e-01,
                 -1.48487675e-01, 2.49910386e-02,
                 -2.21528530e-03, 8.06118266e-05]
        f = poly.Polynomial(coeff)
        green = f(t)
    else:
        coeff = [3.06119745e+00, -6.76337896e-01,
                 8.28276286e-02, -5.72828699e-03,
                 2.35931130e-04, -5.73391101e-06,
                 7.58711054e-08, -4.21266737e-10]
        tt = min(t, 40)
        f = poly.Polynomial(coeff)
        green = f(tt)
    green = max(green, 0)
    green = min(green, 1)

    # calculate blue
    if t < 1.9:
        blue = 0.0
    elif t < 6.6:
        coeff = [4.93997706e-01, -8.59349314e-01,
                 5.45514949e-01, -1.81694167e-01,
                 4.16704799e-02, -6.01602324e-03,
                 4.80731598e-04, -1.61366693e-05]
        f = poly.Polynomial(coeff)
        blue = f(t)
    else:
        blue = 1.0
    blue = max(blue, 0)
    blue = min(blue, 1)

    return (red, green, blue)


def aitoff(lon, lat):
    '''Aitoff projection'''
    lon -= 180
    lon = lon / 180. * math.pi
    lat = lat / 180. * math.pi
    a = np.arccos(np.cos(lat) * np.cos(lon / 2.))
    x = 2.0 * np.cos(lat) * np.sin(lon / 2.) / np.sin(a) * a / 2.
    y = np.sin(lat) / np.sin(a) * a / 2.
    return x, y
